/*  exhaust_malloc.c
 *      -- a program to test the malloc (or safe) debugging
 *  Thomas Padron-McCarthy (padrone@lysator.liu.se), 1992
 *  Latest change of this file: October 3, 1992
 *
 *  This program, "exhaust_malloc", is part of the safe_malloc/debug_malloc
 *  (SM/DM) package for malloc debugging.
 *
 */

#include <stdio.h>
/* #include <malloc.h> */
#include "safe_malloc.h"

/*---------------------------------------------------------------------------*/

extern int errno;
extern char *sys_errlist[];
extern int sys_nerr;

/*  This "malloc_failed" is similar to the standard one in the SMDM
 *  library, except that it prints on stdout instead of stderr,
 *  and that it returns instead of calling "exit".
 */

/*  Yes, I know. I should use varargs or something.
 *  But that's even LESS portable than this well-known kludge!
 */
void malloc_failed(fmt, a1, a2, a3, a4, a5)
  char *fmt, *a1, *a2, *a3, *a4, *a5;
{
    fflush(stdout);
    printf("MALLOC FAILED: ");
    printf(fmt, a1, a2, a3, a4, a5);
    printf("\n");
    fflush(stdout);
} /* malloc_failed */

/*---------------------------------------------------------------------------*/

main(argc, argv)
  int argc;
  char *argv[];
{
    unsigned int chunksize = 1000, nr_chunks = 0, i;
    unsigned long int sum = 0;
    unsigned int largest_chunk = 0;
    char *cp, *largest_chunk_cp = NULL;

    if (argc > 1)
	chunksize = atoi(argv[1]);
    printf("Allocating...\n");
    fflush(stdout);
    while (chunksize > 0) {
	printf("Allocating %d bytes: ", chunksize);
	fflush(stdout);
	if ((cp = safe_malloc(chunksize)) == NULL) {
	    chunksize /= 2;
	}
	else {
	    ++nr_chunks;
	    sum += chunksize;
	    if (chunksize > largest_chunk) {
		largest_chunk = chunksize;
		largest_chunk_cp = cp;
	    }
	    printf("Ok, total = %ld\n", sum);
	    fflush(stdout);

/*
	    for (i = 0; i < chunksize; ++i) {
		cp[i] = ' ' + (i % 25);
	    }

*/

	    chunksize *= 2;
	}
    }
    if (largest_chunk != NULL)
	safe_free(largest_chunk_cp);
    printf("Done. Allocated %d chunks = a total of %d bytes\n",
	   nr_chunks, sum);
}
