/*  safe_malloc.h -- a "checking malloc" package with a debug option
 *  Thomas Padron-McCarthy (padrone@lysator.liu.se), 1991
 *  Latest change of this file: September 24, 1992
 *
 *  This file, "safe_malloc.h", is part of the safe_malloc/debug_malloc
 *  (SM/DM) package for malloc debugging.
 *
 */

#define SAFE_MALLOC

#ifdef DEBUG_MALLOC

	/*  This part is used if we ARE debugging.
	 *  Just keep all the "safe_malloc" etc, and #define DEBUG_MALLOC.
         *  A good place to #define DEBUG_MALLOC is in CFLAGS in the Makefile.
	 */

#ifdef __STDC__
    typedef unsigned int SIZE_T;
    extern void *debug_malloc(SIZE_T s, char *file_name, int line_number);
    extern void *debug_realloc(void *p, SIZE_T s, char *file_name, int line_number);
    extern void *debug_calloc(unsigned int n, SIZE_T s, char *file_name, int line_number);
    extern void debug_free(void *p, char *file_name, int line_number);
#else
    extern char *debug_malloc();
    extern char *debug_realloc();
    extern char *debug_calloc();
    extern void debug_free();
#endif

/*  If we compile with DEBUG_MALLOC, the calls to "safe_malloc" et. al.
 *  will be re-defined as calls to "debug_malloc" et. al.
 */
#define safe_malloc(s)		debug_malloc(s, __FILE__, __LINE__)
#define safe_realloc(p, s)	debug_realloc(p, s, __FILE__, __LINE__)
#define safe_calloc(n, s)	debug_calloc(n, s, __FILE__, __LINE__)
#define safe_free(p)		debug_free(p, __FILE__, __LINE__)

/*  We also re-define the calls to the standard "malloc" et. al.!
 */
#define malloc(s)		debug_malloc(s, __FILE__, __LINE__)
#define realloc(p, s)		debug_realloc(p, s, __FILE__, __LINE__)
#define calloc(n, s)		debug_calloc(n, s, __FILE__, __LINE__)
#define free(p)			debug_free(p, __FILE__, __LINE__)

extern FILE *dm_error_stream;	/* Yes, you must #include <stdio.h> for this to work! */
extern char *dm_dump_file_name;
extern int dm_insane;
extern int dm_repair;
extern int dm_reuse;
extern int dm_fill_new;
extern int dm_fill_freed;
extern int dm_print_codes;
extern char dm_print_char;
extern int dm_max_print_length;
extern char *dm_global_remark;

#ifdef __STDC__
    extern int dm_whatis_pointer(void *p);
    extern int dm_check_all();
    extern void dm_dump();
    extern void dm_full_dump();
#else
    extern int dm_whatis_pointer();
    extern int dm_check_all();
    extern dm_dump();
    extern dm_full_dump();
#endif

#define SET_MALLOC_REMARK(str)	(dm_global_remark = (str))
#define UNSET_MALLOC_REMARK()	SET_MALLOC_REMARK((char *)NULL)

#else

	/* This part is used if we are NOT debugging. */

#ifdef __STDC__
    extern char *safe_malloc(SIZE_T s);
    extern char *safe_calloc(unsigned int n, SIZE_T s);
    extern char *safe_realloc(void *p, SIZE_T s);
    extern void safe_free(void *p);
#else
    extern char *safe_malloc();
    extern char *safe_calloc();
    extern char *safe_realloc();
    extern void safe_free();
#endif

#define malloc(s)		safe_malloc(s)
#define realloc(p, s)		safe_realloc(p, s)
#define calloc(n, s)		safe_calloc(n, s)
#define free(p)			safe_free(p)

#define SET_MALLOC_REMARK(str)	0
#define UNSET_MALLOC_REMARK()	0

#endif
